//---------------------------------------------------------------------------------------------
//                                                                      B A S E D - O N  ----->
//---------------------------------------------------------------------------------------------
// HX711_ADC.h
// Arduino master library for HX711 24-Bit Analog-to-Digital Converter for Weigh Scales
// Olav Kallhovd sept2017
// Tested with      : HX711 asian module on channel A and YZC-133 3kg load cell
// Tested with MCU  : Arduino Nano
//---------------------------------------------------------------------------------------------
// This is an example sketch on how to use this library for two ore more HX711 modules
// Settling time (number of samples) and data filtering can be adjusted in the HX711_ADC.h file

#include <HX711_ADC.h>

#include <EEPROM.h>

#include <LCD.h>
#include <LiquidCrystal_I2C.h>
#include <Wire.h>

LiquidCrystal_I2C  lcd(0x27, 2, 1, 0, 4, 5, 6, 7); // PCF8754 - 0x27, PCF8754A - 0x3F !

// HX711 constructor (dout pin, sck pin)
HX711_ADC LoadCell_1(A1, A0); // HX711 Front LC
HX711_ADC LoadCell_2(A3, A2); // HX711 Rear LC

long t;

int dist;                     // Distance measuring points LCs
int distnew;

int loopcount = 0;            // For stepup/stepdown 1 ... 10 steps
int deltax = 1;

byte value1;                  // For storing dist value in eeprom
byte value2;

void setup() {


  lcd.begin (16, 2);                            // For 16 x 2 LCD module
  lcd.setBacklightPin(3, POSITIVE);
  lcd.setBacklight(HIGH);

  lcd.print("CG - SCALE  v0.8");                // Start message
  lcd.setCursor (0, 1);
  lcd.print("TARE LCs ...wait");

  // Tare preciscion can be improved by adding a few seconds of stabilising time
  // Run startup, stabilization and tare, both modules simultaniously

  LoadCell_1.begin();
  LoadCell_2.begin();
  long stabilisingtime = 6000;
  byte loadcell_1_rdy = 0;
  byte loadcell_2_rdy = 0;
  while ((loadcell_1_rdy + loadcell_2_rdy) < 2) {
    if (!loadcell_1_rdy) loadcell_1_rdy = LoadCell_1.startMultiple(stabilisingtime);
    if (!loadcell_2_rdy) loadcell_2_rdy = LoadCell_2.startMultiple(stabilisingtime);
  }

  LoadCell_1.setCalFactor(1804.0);  // User set calibration factor (float)
  LoadCell_2.setCalFactor(2089.0);  // User set calibration factor (float)


  value1 = EEPROM.read(0);
  value2 = EEPROM.read(1);
  dist = word(value2, value1);      // Get previous LC dist from eeprom
  distnew = dist;


  lcd.clear ();
  lcd.print("LC dist.(eeprom)");
  lcd.setCursor (0, 1);
  lcd.print(dist, DEC);
  lcd.print(" mm |<---->|");
  delay(2000);

  lcd.clear ();


}

void loop() {



  if (digitalRead(2) == HIGH) {         // save value to eeprom
    savedist ();
  }

  while (digitalRead(3) == HIGH) {      // increase value
    increase ();
  }
  while (digitalRead(4) == HIGH) {      // decrease value
    decrease ();
  }


  // Update() should be called at least as often as HX711 sample rate; >10Hz@10SPS, >80Hz@80SPS
  // Longer delay in scetch will reduce effective sample rate (be carefull with delay() in loop)

  LoadCell_1.update();
  LoadCell_2.update();

  // Get smoothed value from data set + current calibration factor

  if (millis() > t + 250) {
    float frontload = LoadCell_1.getData();
    float rearload = LoadCell_2.getData();

    t = millis();


    if (frontload > 10 && rearload > 10) {  // load on?

      int weight = frontload + rearload;
      int cgdist = (distnew * rearload) / (weight);

      lcd.clear ();
      lcd.print("CG <--->  Weight");
      lcd.setCursor (0, 1);
      lcd.print(cgdist);
      lcd.print(" mm    ");
      lcd.print(weight);
      lcd.print(" g");


      delay(100);
    }

    else {
      lcd.clear ();
      lcd.print("Waiting.........");
      delay(100);
    }
  }

  deltax = 1;
  loopcount = 0;

}



void savedist () {
  if (dist != distnew) {
    value1 = highByte(distnew);
    value2 = lowByte(distnew);
    EEPROM.write(0, value2);
    EEPROM.write(1, value1);
    lcd.setCursor (0, 0);
    lcd.print("LC dist.(saved!)");
    lcd.setCursor (0, 1);
    lcd.print(distnew, DEC);
    lcd.print(" mm");
    delay(2500);

  }
}


void increase () {
  distnew = distnew + deltax;
  loopcount = loopcount + 1;      // speed up loop
  if (loopcount > 10) {
    deltax = 10;
  }
  lcd.clear ();
  lcd.setCursor (0, 0);
  lcd.print("LC dist.(eeprom)");
  lcd.setCursor (0, 1);
  lcd.print(distnew, DEC);
  lcd.print(" mm");
  delay(250);
}

void decrease () {
  distnew = distnew - deltax;
  loopcount = loopcount + 1;
  if (loopcount > 10) {
    deltax = 10;
  }
  lcd.clear ();
  lcd.setCursor (0, 0);
  lcd.print("LC dist.(eeprom)");
  lcd.setCursor (0, 1);
  lcd.print(distnew, DEC);
  lcd.print(" mm");
  delay(250);
}


